package com.blogspot.revbingo.threed;

import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.JCheckBox;
import javax.swing.JFrame;

import com.blogspot.revbingo.threed.lights.AmbientLight;
import com.blogspot.revbingo.threed.lights.Light;
import com.blogspot.revbingo.threed.lights.LightScene;
import com.blogspot.revbingo.threed.lights.Spotlight;
import com.blogspot.revbingo.threed.objects.BasicSceneObject;
import com.blogspot.revbingo.threed.objects.Cuboid;
import com.blogspot.revbingo.threed.objects.Spheroid;
import com.blogspot.revbingo.threed.objects.XYPlane;

public class ThreeDee extends JFrame {

	private Random r = new Random(); // used to generate random points
	ArrayList<BasicSceneObject> scene = new ArrayList<BasicSceneObject>();
	Spotlight currentSpotlight;
	Spotlight[] spotlights;
	Point spotPoint = new Point(0, 0, 0);
	final ThreeDeePanel panel;
	final InfoPanel infoPanel = new InfoPanel();

	public enum MoveState {
		MOVE_OBJECT,
		MOVE_LIGHT_POSITION,
		MOVE_LIGHT_DIRECTION,
		MOVE_CAMERA
	}
	
	private MoveState moveState = MoveState.MOVE_OBJECT;
	private boolean stop;
	
	public static void main(String[] args) {
		new ThreeDee();
	}

	public ThreeDee() {
		panel = new ThreeDeePanel();
		panel.setBackground(Color.BLACK);

		createScene(); // create our objects
		final LightScene lights = new LightScene();
		// Add a light to the scene

		//lights.add(new Spotlight(new Color(255,255,150), new Point(0, 0, -150), new Point(0, 0, 0), 20, 15));
		lights.add(new Spotlight(new Color(255,255,255), new Point(100, 100, -150), new Point(0, 0, 0), 10, 10));
		lights.add(new Spotlight(new Color(100,150,255), new Point(-100, 100, -150), new Point(150, -50, 200), 10, 10));
		lights.add(new AmbientLight(new Color(0, 0, 30)));

		currentSpotlight = (Spotlight) lights.get(0); 
		
		infoPanel.setCurrentState(moveState.toString());
		infoPanel.setFps(0);
		infoPanel.setLightDirection(currentSpotlight.getDirection().toString());
		infoPanel.setLightPosition(currentSpotlight.getPosition().toString());
		infoPanel.setPolyCount(0);
		
		panel.setScene(scene, lights);
		panel.setInfoPanel(infoPanel);
		
		RotationMouseListener mouseListener = new RotationMouseListener();
		panel.addMouseWheelListener(mouseListener);
		panel.addMouseMotionListener(mouseListener);
		panel.addMouseListener(mouseListener);
		panel.setFocusable(true);
		panel.addKeyListener(new KeyAdapter() {

			@Override
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_SPACE) {
					MoveState[] states = MoveState.values();
					MoveState nextState = states[(moveState.ordinal() + 1) % 4];
					moveState = nextState;
					infoPanel.setCurrentState(moveState.toString());
					panel.repaint();
				} else if (e.getKeyCode() == KeyEvent.VK_N 
						&& (moveState == MoveState.MOVE_LIGHT_POSITION || moveState == MoveState.MOVE_LIGHT_DIRECTION)) {
					int lightIndex = lights.indexOf(currentSpotlight);
					Light nextLight;
					do {
						lightIndex++;
						nextLight = lights.get(lightIndex % lights.size());
					} while (!( nextLight instanceof Spotlight));
					currentSpotlight = (Spotlight) nextLight;
					infoPanel.setCurrentLight(lightIndex);
					System.out.println(currentSpotlight.getFocus().toString());
					infoPanel.setLightDirection(currentSpotlight.getFocus().toString());
					infoPanel.setLightPosition(currentSpotlight.getPosition().toString());
				}
			}
			
		});
		
		this.add(panel);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setSize(800, 600);
		this.setVisible(true);
		
		while(!stop) {
			panel.repaint();
			//try { Thread.sleep(50); } catch (Exception e) {}
		}
	}

	/**
	 * Creates the 3D scene as a series of drawable objects
	 * 
	 * @return ArrayList<SceneObject> - the scene to be drawn
	 */
	private void createScene() {
		final int DETAIL = 30;
		BasicSceneObject sphere = new Spheroid(100, 40, 40, DETAIL);
		sphere.setColor(new Color(200,200,30));
		scene.add(sphere);

		 BasicSceneObject sphere2 = new Spheroid(40,15,15,DETAIL);
		 sphere2.translate(50, 40, -20);
		 sphere2.setColor(Color.RED);
		 scene.add(sphere2);
		 
		 BasicSceneObject sphere3 = new Spheroid(40,15,15,DETAIL);
		 sphere3.translate(50, -40, -20);
		 sphere3.setColor(Color.RED);
		 scene.add(sphere3);

		BasicSceneObject backWall = new XYPlane(new Point(-200,-200,200), 200, 200, 20);
		scene.add(backWall);
		BasicSceneObject cube = new Cuboid(10, 10, 10);
		cube.translate(-200, 0, 0);
		scene.add(cube);
	}

	public class RotationMouseListener extends MouseMotionAdapter implements
			MouseWheelListener, MouseListener {

		private int oldX;
		private int oldY;

		private final double RADIAN = Math.PI / 180;
		private double anglePerClick = 3 * RADIAN; // a scaling factor for
													// scroll wheel

		
		/*
		 * If the scroll wheel is moved, rotate around the Z axis
		 */
		@Override
		public void mouseWheelMoved(MouseWheelEvent e) {
			int mods = e.getModifiersEx();					
			int dz = e.getWheelRotation();//* e.getClickCount();
			switch(moveState) {
			case MOVE_OBJECT:
				RotationMatrix zRot = RotationMatrix.getInstance(anglePerClick * dz, RotationAxis.Z);
				for (SceneObject d : scene) {
					d.rotate(zRot);
				}
				break;
			case MOVE_LIGHT_POSITION:
				break;
			case MOVE_LIGHT_DIRECTION:
				if ((mods & MouseEvent.CTRL_DOWN_MASK) == MouseEvent.CTRL_DOWN_MASK) {					
					//change falloff radius
					currentSpotlight.setFullIntensityAngle(currentSpotlight.getFullIntensityAngle() + dz);
				} else {
					currentSpotlight.setFalloffAngle(currentSpotlight.getFalloffAngle() + dz);
				}
			}
		}

		/*
		 * When the mouse is dragged, rotate the scene dependent upon delta in
		 * x/y
		 */
		public void mouseDragged(MouseEvent e) {
			int x = e.getX();
			int y = e.getY();
			int dx = oldX - x;
			int dy = oldY - y;
			oldX = x;
			oldY = y;
			
			int mods = e.getModifiersEx();					
			switch(moveState) {
			case MOVE_OBJECT:
				double xangle = dx * RADIAN;
				double yangle = dy * RADIAN;
				RotationMatrix yRot = RotationMatrix.getInstance(xangle, RotationAxis.Y);
				RotationMatrix xRot = RotationMatrix.getInstance(yangle, RotationAxis.X);
				RotationMatrix totalRot = xRot.times(yRot);
				for (SceneObject d : scene) {
					d.rotate(totalRot);
				}
				break;
			case MOVE_LIGHT_DIRECTION:
				if ((mods & MouseEvent.CTRL_DOWN_MASK) == MouseEvent.CTRL_DOWN_MASK) {					
					spotPoint.z += dy;
				} else {
					spotPoint.x -= dx;
					spotPoint.y += dy;
					
				}
				currentSpotlight.setFocus(spotPoint);
				infoPanel.setLightDirection(spotPoint.toString());
				break;
			case MOVE_LIGHT_POSITION:
				Point pos = currentSpotlight.getPosition();
				if ((mods & MouseEvent.CTRL_DOWN_MASK) == MouseEvent.CTRL_DOWN_MASK) {					
					pos.z += dy;
				} else {
					pos.x -= dx;
					pos.y += dy;
				}
				currentSpotlight.setPosition(pos);
				infoPanel.setLightPosition(pos.toString());
			default:
				
			}
		}

		/*
		 * Even if the mouse button isn't pressed, keep track of the position so
		 * that the object doesn't "jump" when we press the mouse again
		 */
		public void mouseMoved(MouseEvent e) {
			oldX = e.getX();
			oldY = e.getY();
		}

		/*
		 * Clicking the mouse switches between wireframe and shaded
		 */
		@Override
		public void mouseClicked(MouseEvent e) {
			Triangle.WIREFRAME = !Triangle.WIREFRAME;
		}

		@Override
		public void mouseEntered(MouseEvent e) {}
		@Override
		public void mouseExited(MouseEvent e) {}
		@Override
		public void mousePressed(MouseEvent e) {}
		@Override
		public void mouseReleased(MouseEvent e) {}		
	}
}
