package com.blogspot.revbingo.threed.objects;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Collection;

import com.blogspot.revbingo.threed.Primitive;
import com.blogspot.revbingo.threed.RotationAxis;
import com.blogspot.revbingo.threed.RotationMatrix;
import com.blogspot.revbingo.threed.SceneObject;

public abstract class BasicSceneObject extends SceneObject {

	//List of polygons in this object
	protected Collection<Primitive> object = new ArrayList<Primitive>();
	
	//keep track of the position of the object in world space
	private double offsetX;
	private double offsetY;
	private double offsetZ;
	
	@Override
	public void draw(Graphics2D g) {
		for(Primitive polygon : object) {
			polygon.draw(g);
		}
	}

	public void translate(double dx, double dy, double dz) {
		offsetX += dx;
		offsetY += dy;
		offsetZ += dz;
		for(Primitive polygon : object) {
			polygon.translate(dx, dy, dz);
		}
	}
	
	public void rotate(double radians, RotationAxis axis) {
		RotationMatrix rotMat = RotationMatrix.getInstance(radians, axis);
		rotate(rotMat);
	}
	
	public void rotate(RotationMatrix rotMat) {
		for(Primitive polygon : object) {
			polygon.rotate(rotMat);
		}
	}
	
	public Collection<Primitive> getPolygons() {
		return object;
	}
	
	public void setColor(Color c) {
		for(Primitive p : object) {
			p.setSurfaceColor(c);
		}
	}
}
